from django.test import TestCase

from wagtail.wagtailimages.tests.utils import Image, get_test_image_file

from molo.core.models import (
    ArticlePage,
    SectionPage,
    Tag,
    FooterPage,
    BannerPage,
)
from molo.core.api.constants import ERROR
from molo.core.api.tests import constants
from molo.core.api import importers
from molo.core.tests.base import MoloTestCaseMixin


class TestImportableMixin(MoloTestCaseMixin, TestCase):
    def setUp(self):
        self.mk_main()

    def test_tag_importable(self):
        content = constants.TAG_PAGE_RESPONSE
        content_copy = dict(content)
        class_ = Tag

        tag = Tag.create_page(content_copy, class_)

        self.assertEqual(type(tag), Tag)
        self.assertEqual(tag.title, content["title"])
        self.assertEqual(tag.title, content["title"])
        self.assertEqual(tag.feature_in_homepage,
                         content["feature_in_homepage"])
        self.assertEqual(tag.go_live_at, content["go_live_at"])
        self.assertEqual(tag.expire_at, content["expire_at"])
        self.assertEqual(tag.expired, content["expired"])

    def check_article_and_footer_fields(self, page, content, record_keeper):
        self.assertEqual(page.title, content["title"])
        self.assertEqual(page.subtitle, content["subtitle"])
        self.assertEqual(page.commenting_state, content["commenting_state"])
        self.assertEqual(page.social_media_title,
                         content["social_media_title"])
        self.assertEqual(page.social_media_description,
                         content["social_media_description"])
        self.assertEqual(page.featured_in_latest,
                         content["featured_in_latest"])
        self.assertEqual(page.featured_in_section,
                         content["featured_in_section"])
        self.assertEqual(page.featured_in_homepage,
                         content["featured_in_homepage"])
        self.assertEqual(page.feature_as_topic_of_the_day,
                         content["feature_as_topic_of_the_day"])

        self.assertEqual(page.commenting_open_time,
                         content["commenting_open_time"])
        self.assertEqual(page.commenting_close_time,
                         content["commenting_close_time"])
        self.assertEqual(page.featured_in_latest_start_date,
                         content["featured_in_latest_start_date"])
        self.assertEqual(page.featured_in_latest_end_date,
                         content["featured_in_latest_end_date"])
        self.assertEqual(page.featured_in_section_start_date,
                         content["featured_in_section_start_date"])
        self.assertEqual(page.featured_in_section_end_date,
                         content["featured_in_section_end_date"])
        self.assertEqual(page.featured_in_homepage_start_date,
                         content["featured_in_homepage_start_date"])
        self.assertEqual(page.featured_in_homepage_end_date,
                         content["featured_in_homepage_end_date"])
        self.assertEqual(page.promote_date,
                         content["promote_date"])
        self.assertEqual(page.demote_date,
                         content["demote_date"])

        # NESTED FIELDS
        self.assertTrue(hasattr(page.body, "stream_data"))
        self.assertEqual(page.body.stream_data, content['body'])

        self.assertEqual(page.tags.count(), len(content['tags']))
        for tag in page.tags.all():
            self.assertTrue(tag.name in content["tags"])

        self.assertEqual(page.metadata_tags.count(),
                         len(content['metadata_tags']))
        for metadata_tag in page.metadata_tags.all():
            self.assertTrue(metadata_tag.name in content["metadata_tags"])

        # Check that image has been added
        self.assertTrue(page.image)
        self.assertEqual(page.image.title, content["image"]["title"])
        # Check that social media file has been added
        self.assertTrue(page.social_media_image)
        self.assertEqual(page.social_media_image.title,
                         content["social_media_image"]["title"])

        # Check that foreign relationships have been created
        self.assertTrue(content["id"] in
                        record_keeper.foreign_to_many_foreign_map["nav_tags"])
        self.assertEqual((record_keeper.foreign_to_many_foreign_map["nav_tags"]
                          [content["id"]]),
                         [content["nav_tags"][0]["tag"]["id"], ])

        self.assertTrue(
            content["id"] in
            record_keeper.foreign_to_many_foreign_map["recommended_articles"])
        self.assertEqual(
            (record_keeper.foreign_to_many_foreign_map["recommended_articles"]
                [content["id"]]),
            [content["recommended_articles"][0]["recommended_article"]["id"],
             content["recommended_articles"][1]["recommended_article"]["id"]])

        self.assertTrue(
            content["id"] in
            record_keeper.foreign_to_many_foreign_map["reaction_questions"])
        self.assertEqual(
            (record_keeper.foreign_to_many_foreign_map["reaction_questions"]
                [content["id"]]),
            [content["reaction_questions"][0]["reaction_question"]["id"],
             content["reaction_questions"][1]["reaction_question"]["id"]])

        self.assertTrue(
            content["id"] in
            record_keeper.foreign_to_many_foreign_map["related_sections"])
        self.assertEqual(
            (record_keeper.foreign_to_many_foreign_map["related_sections"]
                [content["id"]]),
            [content["related_sections"][0]["section"]["id"],
             content["related_sections"][1]["section"]["id"]])

    def test_article_importable(self):
        content = constants.ARTICLE_PAGE_RESPONSE
        content_copy = dict(content)

        # Validate Assumptions
        #   The images have already been imported
        #   The record keeper has mapped the relationship

        foreign_image_id = content["image"]["id"]
        image = Image.objects.create(
            title=content["image"]["title"],
            file=get_test_image_file(),
        )

        foreign_social_media_image_id = content["social_media_image"]["id"]
        social_media_image = Image.objects.create(
            title=content["social_media_image"]["title"],
            file=get_test_image_file(),
        )

        record_keeper = importers.RecordKeeper()
        record_keeper.record_image_relation(foreign_image_id, image.id)
        record_keeper.record_image_relation(
            foreign_social_media_image_id, social_media_image.id)

        class_ = ArticlePage

        page = ArticlePage.create_page(
            content_copy, class_, record_keeper=record_keeper)

        self.check_article_and_footer_fields(page, content, record_keeper)

    def test_article_body_not_importable(self):
        content = constants.ARTICLE_PAGE_RESPONSE_STREAM_FIELDS
        content_copy = dict(content)

        record_keeper = importers.RecordKeeper()

        class_ = ArticlePage

        page = ArticlePage.create_page(
            content_copy, class_, record_keeper=record_keeper)

        self.assertEqual(page.title, content["title"])
        self.assertFalse(page.body)

        self.assertTrue(content["id"] in record_keeper.article_bodies)
        self.assertEqual(
            record_keeper.article_bodies[content["id"]],
            content["body"])

    def test_section_importable(self):
        content = constants.SECTION_PAGE_RESPONSE
        content_copy = dict(content)

        # Validate Assumptions
        #   The images have already been imported
        #   The record keeper has mapped the relationship

        foreign_image_id = content["image"]["id"]
        image = Image.objects.create(
            title=content["image"]["title"],
            file=get_test_image_file(),
        )

        record_keeper = importers.RecordKeeper()
        record_keeper.record_image_relation(foreign_image_id, image.id)

        class_ = SectionPage

        page = SectionPage.create_page(
            content_copy, class_, record_keeper=record_keeper)

        self.assertEqual(page.title, content["title"])
        self.assertEqual(page.description, content["description"])
        self.assertEqual(page.extra_style_hints,
                         content["extra_style_hints"])
        self.assertEqual(page.commenting_state, content["commenting_state"])
        self.assertEqual(page.monday_rotation, content["monday_rotation"])
        self.assertEqual(page.tuesday_rotation, content["tuesday_rotation"])
        self.assertEqual(page.wednesday_rotation,
                         content["wednesday_rotation"])
        self.assertEqual(page.thursday_rotation,
                         content["thursday_rotation"])
        self.assertEqual(page.friday_rotation, content["friday_rotation"])
        self.assertEqual(page.saturday_rotation,
                         content["saturday_rotation"])
        self.assertEqual(page.sunday_rotation, content["sunday_rotation"])

        self.assertEqual(page.commenting_open_time,
                         content["commenting_open_time"])
        self.assertEqual(page.commenting_close_time,
                         content["commenting_close_time"])
        self.assertEqual(page.content_rotation_start_date,
                         content["content_rotation_start_date"])
        self.assertEqual(page.content_rotation_end_date,
                         content["content_rotation_end_date"])

        # NESTED FIELDS
        self.assertTrue(hasattr(page.time, "stream_data"))
        self.assertEqual(page.time.stream_data, content["time"])

        # Check that image has been added
        self.assertTrue(page.image)
        self.assertEqual(page.image.title, content["image"]["title"])

        # Check that foreign relationships have been created
        self.assertTrue(
            content["id"] in
            record_keeper.foreign_to_many_foreign_map["section_tags"])
        self.assertEqual(
            (record_keeper.foreign_to_many_foreign_map["section_tags"]
                [content["id"]]),
            [content["section_tags"][0]["tag"]["id"],
             content["section_tags"][1]["tag"]["id"]])

    def test_footer_importable(self):
        content = constants.ARTICLE_PAGE_RESPONSE
        content_copy = dict(content)

        # Validate Assumptions
        #   The images have already been imported
        #   The record keeper has mapped the relationship

        foreign_image_id = content["image"]["id"]
        image = Image.objects.create(
            title=content["image"]["title"],
            file=get_test_image_file(),
        )

        foreign_social_media_image_id = content["social_media_image"]["id"]
        social_media_image = Image.objects.create(
            title=content["social_media_image"]["title"],
            file=get_test_image_file(),
        )

        record_keeper = importers.RecordKeeper()
        record_keeper.record_image_relation(foreign_image_id, image.id)
        record_keeper.record_image_relation(
            foreign_social_media_image_id, social_media_image.id)

        class_ = FooterPage

        page = FooterPage.create_page(
            content_copy, class_, record_keeper=record_keeper)

        self.check_article_and_footer_fields(page, content, record_keeper)

    def test_banner_importable(self):
        content = constants.BANNER_PAGE_RESPONSE
        content_copy = dict(content)

        # Validate Assumptions
        #   The images have already been imported
        #   The record keeper has mapped the relationship

        foreign_image_id = content["banner"]["id"]
        image = Image.objects.create(
            title=content["banner"]["title"],
            file=get_test_image_file(),
        )

        record_keeper = importers.RecordKeeper()
        record_keeper.record_image_relation(foreign_image_id, image.id)

        class_ = BannerPage

        page = BannerPage.create_page(
            content_copy, class_, record_keeper=record_keeper)

        self.assertEqual(page.title, content["title"])
        self.assertEqual(page.external_link, content["external_link"])

        # check that banner link has been created
        self.assertEqual(
            (record_keeper.foreign_to_foreign_map["banner_link_page"]
                [content["id"]]),
            content["banner_link_page"]["id"])

        # check that banner image has been attached
        self.assertTrue(page.banner)
        self.assertEqual(page.banner, image)
        self.assertEqual(page.banner.title, content["banner"]["title"])

    def test_image_not_imported_logs_error(self):
        content = constants.ARTICLE_WITH_ONLY_IMAGE_RESPONSE
        content_copy = dict(content)

        # Break the assumptions that the images have already
        # been imported thus the record keeper has mapped
        # the relationship

        record_keeper = importers.RecordKeeper()
        logger = importers.Logger()

        class_ = ArticlePage

        ArticlePage.create_page(
            content_copy, class_,
            record_keeper=record_keeper,
            logger=logger)

        self.assertTrue(len(logger.record), 1)
        error_log = logger.record[0]
        self.assertEqual(error_log["log_type"], ERROR)



class PageEffectiveImageMixin(object):
    def get_effective_image(self):
        if self.image:
            return self.image
        page = self.get_main_language_page()
        if page.specific.image:
            return page.specific.get_effective_image()
        return ''



from django.conf.urls import url

from molo.core.admin import (
    ReactionQuestionsModelAdmin, ReactionQuestionsSummaryModelAdmin,
    AdminViewGroup
)
from molo.core.admin_views import ReactionQuestionResultsAdminView, \
    ReactionQuestionSummaryAdminView
from molo.core.models import LanguageRelation, PageTranslation, Languages
from molo.core.utils import create_new_article_relations


from django.core import urlresolvers
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.utils.html import format_html

from wagtail.wagtailcore import hooks
from wagtail.wagtailcore.models import Page
from wagtail.wagtailadmin.menu import MenuItem
from wagtail.wagtailadmin.site_summary import SummaryItem
from wagtail.wagtailadmin.widgets import Button, ButtonWithDropdownFromHook
from wagtail.contrib.modeladmin.options import modeladmin_register
from wagtail.wagtailadmin.wagtail_hooks import page_listing_more_buttons

from . import views

from molo.core.api import urls as molo_api_urls


@hooks.register('register_admin_urls')
def urlconf_translations():
    return [
        url(
            r'^translations/add/(?P<page_id>\d+)/(?P<locale>[\w\-\_]+)/$',
            'molo.core.views.add_translation',
            name='add_translation'),
    ]


@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^import-git/$',
            views.import_from_git,
            name='import-from-git'),
    ]


@hooks.register('register_admin_urls')
def register_question_results_admin_view_url():
    return [
        url(r'reactionquestion/(?P<parent>\d+)/results/$',
            ReactionQuestionResultsAdminView.as_view(),
            name='reaction-question-results-admin'),
    ]


modeladmin_register(ReactionQuestionsModelAdmin)


@hooks.register('register_admin_urls')
def register_article_question_results_admin_view_url():
    return [
        url(r'reactionquestion/(?P<article>\d+)/results/summary/$',
            ReactionQuestionSummaryAdminView.as_view(),
            name='reaction-question-article-results-admin'),
    ]


modeladmin_register(ReactionQuestionsSummaryModelAdmin)
modeladmin_register(AdminViewGroup)


@hooks.register('construct_explorer_page_queryset')
def show_main_language_only(parent_page, pages, request):
    main_language = Languages.for_site(request.site).languages.filter(
        is_main_language=True).first()
    if main_language and parent_page.depth > 2:
        return pages.filter(languages__language__locale=main_language.locale)
    return pages


@hooks.register('after_copy_page')
def add_new_tag_article_relations(request, page, new_page):
    if new_page.depth < 3:
        create_new_article_relations(page, new_page)


@hooks.register('after_copy_page')
def copy_translation_pages(request, page, new_page):

    # Only copy translations for TranslatablePageMixin
    if not hasattr(page.specific, 'copy_language'):
        return 'Not translatable page'

    current_site = page.get_site()
    destination_site = new_page.get_site()
    if current_site is not destination_site and (page.depth > 2):
        page.specific.copy_language(current_site, destination_site)
    languages = Languages.for_site(destination_site).languages
    if (languages.filter(is_main_language=True).exists() and
            not new_page.languages.exists()):
        LanguageRelation.objects.create(
            page=new_page,
            language=languages.filter(
                is_main_language=True).first())

    for translation in page.translations.all():
        new_lang = translation.translated_page.specific.copy_language(
            current_site, destination_site)
        new_translation = translation.translated_page.copy(
            to=new_page.get_parent())
        new_l_rel, _ = LanguageRelation.objects.get_or_create(
            page=new_translation)
        new_l_rel.language = new_lang
        new_l_rel.save()
        PageTranslation.objects.create(
            page=new_page,
            translated_page=new_translation)


@hooks.register('register_admin_menu_item')
def register_import_menu_item():
    return MenuItem(
        _('Import content'),
        urlresolvers.reverse('import-from-git'),
        classnames='icon icon-download',
    )


# API admin
@hooks.register("register_admin_urls")
def add_import_view():
    return molo_api_urls.urlpatterns


@hooks.register('register_admin_menu_item')
def register_api_menu_item():
    return MenuItem(
        _('API'),
        urlresolvers.reverse('site-import'),
        classnames='icon icon-download',
    )


class LanguageSummaryItem(SummaryItem):
    order = 500
    template = 'wagtail/site_languages_summary.html'

    def get_context(self):
        languages = Languages.for_site(self.request.site).languages.all()
        return {
            'summaries': [{
                'language': l.get_locale_display(),
                'total': Page.objects.filter(
                    languages__language__id=l.id).count()
            }for l in languages],
        }


@hooks.register('construct_homepage_summary_items')
def add_languages_summary_item(request, items):
    items.append(LanguageSummaryItem(request))


class LanguageErrorMessage(SummaryItem):
    order = 100
    template = 'wagtail/language_error_message.html'


@hooks.register('construct_homepage_panels')
def add_language_error_message_panel(request, panels):
    if not Languages.for_site(request.site).languages.all().exists():
        panels[:] = [LanguageErrorMessage(request)]


@hooks.register('construct_main_menu')
def hide_menu_items_if_no_language(request, menu_items):
    if not Languages.for_site(request.site).languages.all().exists():
        menu_items[:] = [
            item for item in menu_items if (
                item.name == 'settings' or
                item.name == 'import-content' or
                item.name == 'api')]


@hooks.register('construct_main_menu')
def hide_import_content_if_not_uc_user(request, menu_items):
    if not User.objects.filter(
            pk=request.user.pk,
            groups__name='Universal Core Importers').exists():
        menu_items[:] = [
            item for item in menu_items if item.name != 'import-content']


@hooks.register('construct_main_menu')
def hide_site_import_if_not_in_importer_group(request, menu_items):
    if not User.objects.filter(
            pk=request.user.pk,
            groups__name='Site Importers').exists():
        menu_items[:] = [
            item for item in menu_items if item.name != 'api']


@hooks.register('construct_main_menu')
def show_explorer_only_to_users_have_access(request, menu_items):
    if (request.user.is_superuser or
        User.objects.filter(pk=request.user.pk, groups__name__in=[
            'Moderator', 'Editor']).exists()):
        return menu_items
    if User.objects.filter(pk=request.user.pk, groups__name__in=[
            'Comment Moderator', 'Expert', 'Wagtail Login Only']).exists():
        menu_items[:] = [
            item for item in menu_items if item.name != 'explorer']


@hooks.register('register_page_listing_buttons')
def page_custom_listing_buttons(page, page_perms, is_parent=False):
    yield ButtonWithDropdownFromHook(
        'More',
        hook_name='my_button_dropdown_hook',
        page=page,
        page_perms=page_perms,
        is_parent=is_parent,
        priority=50
    )


@hooks.register('register_page_listing_more_buttons')
def page_listing_buttons(page, page_perms, is_parent=False):
    """
    This removes the standard wagtail dropdown menu.

    This supresses the original 'More' dropdown menu because it breaks
    the expected behaviour of the yeild functionality used to add
    additional buttons in wagtail_hooks.
    """
    if page_perms.can_move():
        return None


@hooks.register('my_button_dropdown_hook')
def new_page_listing_buttons(page, page_perms, is_parent=False):
    """
    This inherits the buttons from wagtail's page_listing_more_buttons
    https://github.com/wagtail/wagtail/blob/stable/1.8.x/wagtail/wagtailadmin/wagtail_hooks.py#L94
    (i.e. the buttons that are put in the original drop down menu)
    This is done to avoid breakages should their hooks change in the future


    It iterates through the buttons and prevents the delete button
    from being added if the Page should not be deleteable from the admin UI
    """
    original_buttons = list(page_listing_more_buttons(page,
                                                      page_perms,
                                                      is_parent))
    if not hasattr(page.specific, 'hide_delete_button'):
        for b in original_buttons:
            yield b
    else:
        for b in original_buttons:
            if (hasattr(b, 'attrs') and
                    'delete' not in b.attrs.get('title').lower()):
                    yield b

    if not page_perms.can_unpublish():
        yield Button(
            _('Publish'),
            urlresolvers.reverse('publish', args=(page.id,)),
            attrs={'title': _("Publish page '{title}'").format(
                title=page.get_admin_display_title()
            )},
            priority=40
        )


@hooks.register('insert_global_admin_css')
def global_admin_css():
    return format_html(
        '<link rel="stylesheet" href="{}">', static('css/wagtail-admin.css'))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


def delete_imageinfo(apps, schema_editor):
    ImageInfo = apps.get_model('core.ImageInfo')
    ImageInfo.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0070_add_service_directory_api'),
    ]

    operations = [
        migrations.RunPython(delete_imageinfo),
    ]



from django.test import TestCase
from django.contrib.auth.models import User
from wagtail.wagtailimages.tests.utils import get_test_image_file, Image

from molo.core.models import (
    Main, SiteLanguageRelation, Languages,
    ArticlePage)
from molo.core.tests.base import MoloTestCaseMixin


class TestAdminView(TestCase, MoloTestCaseMixin):
    def setUp(self):
        self.mk_main()
        main = Main.objects.all().first()
        self.english = SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(main.get_site()),
            locale='en',
            is_active=True)

        self.french = SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(main.get_site()),
            locale='fr',
            is_active=True)

        self.yourmind = self.mk_section(
            self.section_index, title='Your Mind')

        self.yourmind_fr = self.mk_section_translation(
            self.yourmind, self.french, title='Your mind in french')

        self.mk_main2()
        self.main2 = Main.objects.all().last()
        self.language_setting2 = Languages.objects.create(
            site_id=self.main2.get_site().pk)
        self.english2 = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting2,
            locale='en',
            is_active=True)

        self.spanish = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting2,
            locale='es',
            is_active=True)

        self.yourmind2 = self.mk_section(
            self.section_index2, title='Your mind2')
        User.objects.create_superuser(
            username='testuser', password='password', email='test@email.com')
        self.client.login(username='testuser', password='password')

    def test_articles_appears_in_admin_view(self):
        self.mk_article(self.yourmind)
        response = self.client.get(
            '/admin/core/articlepagelanguageproxy/'
        )
        self.assertContains(response, 'Test page 0')

    def test_section_in_admin_view(self):
        self.mk_article(self.yourmind)
        response = self.client.get(
            '/admin/core/articlepagelanguageproxy/'
        )
        self.assertContains(response, 'Your Mind')

    def test_article_tag_in_admin_view(self):
        article = self.mk_article(self.yourmind, title='article')
        article.tags.add("the tag")
        article.save_revision().publish()
        response = self.client.get(
            '/admin/core/articlepagelanguageproxy/'
        )
        self.assertContains(response, 'the tag')

    def test_article_image_in_admin_view(self):
        self.image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        article = self.mk_article(self.yourmind,
                                  title='article',
                                  image=self.image
                                  )
        article.save_revision().publish()
        response = self.client.get(
            '/admin/core/articlepagelanguageproxy/'
        )
        self.assertContains(response, '<img src="/media/images/')

    def test_status_custom_filter_published_in_admin_view(self):
        self.mk_articles(self.yourmind)
        self.article2 = ArticlePage.objects.get(title="Test page 1")
        self.article2.unpublish()
        response = self.client.get(
            '/admin/core/articlepagelanguageproxy/?status=published'
        )
        self.assertContains(response, 'Test page 0')
        self.assertNotContains(response, 'Test page 1')

    def test_status_custom_filter_in_review_in_admin_view(self):
        self.mk_articles(self.yourmind)
        self.article2 = ArticlePage.objects.get(title="Test page 1")
        self.article2.save_revision(submitted_for_moderation=True)
        response = self.client.get(
            '/admin/core/articlepagelanguageproxy/?status=in_review'
        )
        self.assertNotContains(response, 'Test page 0')
        self.assertContains(response, 'Test page 1')

    def test_status_custom_filter_draft_in_admin_view(self):
        self.mk_articles(self.yourmind)
        self.article2 = ArticlePage.objects.get(title="Test page 1")
        self.article2.unpublish()
        response = self.client.get(
            '/admin/core/articlepagelanguageproxy/?status=draft'
        )
        self.assertNotContains(response, 'Test page 0')
        self.assertContains(response, 'Test page 1')

    def test_status_custom_filter_sections_in_admin_view(self):
        self.mk_articles(self.yourmind)
        self.article2 = ArticlePage.objects.get(title="Test page 1")
        self.article2.unpublish()
        response = self.client.get(
            '/admin/core/articlepagelanguageproxy/?section=%d' %
            self.yourmind.id
        )
        self.assertContains(response, 'Test page 0')
        self.assertContains(response, 'Test page 1')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-07-20 12:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0040_auto_20160722_1041'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='global_ga_tracking_code',
            field=models.CharField(blank=True, help_text='Global GA tracking code to be used to view analytics on more than one site globally', max_length=255, null=True, verbose_name='Global GA Tracking Code'),
        ),
        migrations.AddField(
            model_name='sitesettings',
            name='local_ga_tracking_code',
            field=models.CharField(blank=True, help_text='Local GA tracking code to be used to view analytics on this site only', max_length=255, null=True, verbose_name='Local GA Tracking Code'),
        ),
    ]



import requests
from mock import patch

from django.test import TestCase

from wagtail.wagtailimages.models import Image
from wagtail.wagtailimages.tests.utils import get_test_image_file

from molo.core.tests.base import MoloTestCaseMixin
from molo.core.api import forms, importers
from molo.core.api.constants import MAIN_IMPORT_FORM_MESSAGES
from molo.core.api.tests import constants
from molo.core.api.tests.utils import mocked_requests_get
from molo.core.models import ArticlePage
from molo.core.models import SectionPage  # noqa


class ArticleImportFormTestCase(MoloTestCaseMixin, TestCase):

    def setUp(self):
        self.mk_main()

        # content must be list of articles as received from the API endpoint
        self.importer = importers.ArticlePageImporter(
            base_url="http://localhost:8000",
            content=constants.AVAILABLE_ARTICLES["items"]
        )
        self.section = self.mk_section(
            self.section_index,
            title="Parent Section 1",
        )

    def test_article_render_as_field_choices(self):
        form = forms.ArticleImportForm(
            importer=self.importer,
            parent=self.section.id
        )
        self.assertEqual(
            len(form.fields),
            len(constants.AVAILABLE_ARTICLES["items"])
        )

    @patch("molo.core.api.importers.get_image")
    def test_selected_articles_can_be_saved(self, mock_image):
        image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )
        mock_image.return_value = image
        form_data = {
            "0": True,
            "1": True,
            "2": False,
        }
        form = forms.ArticleImportForm(
            data=form_data,
            importer=self.importer,
            parent=self.section.id
        )
        self.assertTrue(form.is_valid())

        # form.save() saves the selected articles and returns the importer
        form.save()
        self.assertEqual(ArticlePage.objects.all().count(), 2)

    def tearDown(self):
        del self.importer


class MainImportFormTestCase(MoloTestCaseMixin, TestCase):

    def setUp(self):
        self.mk_main()

    @patch("molo.core.api.forms.requests.get", side_effect=mocked_requests_get)
    def test_valid_url_is_given(self, mock_get):
        form_data = {
            "url": "http://localhost:8000/api/v2/pages",
            "content_type": "core.ArticlePage"
        }
        form = forms.MainImportForm(
            data=form_data
        )
        self.assertTrue(form.is_valid())

    @patch("molo.core.api.forms.requests.get",
           side_effect=requests.ConnectionError)
    def test_invalid_url_raises_connection_error(self, mock_get):
        form_data = {
            "url": "http://localhost:8000/api/v2/pages",
            "content_type": "core.ArticlePage"
        }
        form = forms.MainImportForm(
            data=form_data
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(
            [MAIN_IMPORT_FORM_MESSAGES["connection_error"], ],
            form.errors["url"]
        )

    @patch("molo.core.api.forms.requests.get",
           side_effect=requests.RequestException)
    def test_bad_url_raises_requests_error(self, mock_get):
        form_data = {
            "url": "http://localhost:8000/api/v2/pages",
            "content_type": "core.ArticlePage"
        }
        form = forms.MainImportForm(
            data=form_data
        )
        self.assertFalse(form.is_valid())
        self.assertEqual(
            [MAIN_IMPORT_FORM_MESSAGES["bad_request"], ],
            form.errors["url"]
        )


class SectionImportFormTestCase(MoloTestCaseMixin, TestCase):

    def setUp(self):
        self.mk_main()
        self.importer = importers.SectionPageImporter(
            base_url="http://localhost:8000",
            content=constants.AVAILABLE_SECTIONS["items"]
        )

    def test_sections_render_as_field_choices(self):
        form = forms.SectionImportForm(
            importer=self.importer,
            parent=self.section_index.id
        )
        self.assertEqual(
            len(form.fields),
            len(constants.AVAILABLE_SECTIONS["items"])
        )

    # @patch("molo.core.api.importers.requests.get",
    #        side_effect=mocked_requests_get)
    # @patch("molo.core.api.importers.get_image")
    # def test_selected_section_can_be_saved(self, mock_image, mock_get):
    #     image = Image.objects.create(
    #         title="Test image",
    #         file=get_test_image_file(),
    #     )
    #     mock_image.return_value = image
    #     form_data = {
    #         "2": True,
    #         "3": False,
    #         "4": False,
    #     }
    #     form = forms.SectionImportForm(
    #         data=form_data,
    #         importer=self.importer,
    #         parent=self.section_index.id
    #     )
    #     self.assertTrue(form.is_valid())
    #
    #     # form.save() saves the selected articles and returns the importer
    #     form.save()
    #     self.assertEqual(ArticlePage.objects.all().count(), 1)
    #     self.assertEqual(SectionPage.objects.all().count(), 2)

    def tearDown(self):
        del self.importer



import csv
import random
import logging

from io import BytesIO
from datetime import datetime

from celery import task
from time import strptime

from django.conf import settings
from django.core.mail import EmailMessage
from django.template.loader import render_to_string
from django.core import management
from django.contrib.auth.models import User

from molo.core.content_import import api
from molo.core.utils import create_new_article_relations
from molo.core.models import (
    Site,
    ArticlePage,
    Main,
    SectionIndexPage,
    SectionPage,
    Languages,
    SiteSettings,
    BannerIndexPage,
    FooterIndexPage,
    TagIndexPage,
)
from molo.core.api.importers import (
    RecordKeeper,
    LanguageImporter,
    ImageImporter,
    ContentImporter,
    Logger,
)
from molo.core.api.constants import ACTION

from django.utils import timezone

from wagtail.wagtailcore.models import Page


IMPORT_EMAIL_TEMPLATE = "content_import/import_email.html"
VALIDATE_EMAIL_TEMPLATE = "content_import/validate_email.html"
COPY_EMAIL_TEMPLATE = "core/copy_email.html"
COPY_FAILED_EMAIL_TEMPLATE = "core/copy_failed_email.html"


@task(ignore_result=True)
def rotate_content(day=None):
    """ this method gets the parameters that are needed for rotate_latest
    and rotate_featured_in_homepage methods, and calls them both"""
    # getting the content rotation settings from site settings

    for main in Main.objects.all():
        site = main.sites_rooted_here.all().first()
        main_lang = Languages.for_site(site).languages.filter(
            is_main_language=True).first()
        index = SectionIndexPage.objects.live().child_of(main).first()
        site_settings = SiteSettings.for_site(site)
        if day is None:
            day = datetime.today().weekday()

        # calls the two rotate methods with the necessary params
        if main and index:
            rotate_latest(main_lang, index, main, site_settings, day)
            rotate_featured_in_homepage(main_lang, day, main)


@task(ignore_result=True)
def publish_scheduled_pages():
    management.call_command(
        'publish_scheduled_pages', verbosity=0, interactive=False)


@task(ignore_result=True)
def clearsessions():
    # Expired sessions will only be cleared roughly once an hour - randomly
    if random.randint(0, 59) == 0:
        management.call_command(
            'clearsessions', verbosity=0, interactive=False)


@task(ignore_result=True)
def demote_articles():
    ArticlePage.objects.live().filter(
        featured_in_latest_end_date__lte=datetime.now()).update(
            featured_in_latest=False,
            featured_in_latest_start_date=None,
            featured_in_latest_end_date=None)
    ArticlePage.objects.live().filter(
        featured_in_section_end_date__lte=datetime.now()).update(
            featured_in_section=False,
            featured_in_section_start_date=None,
            featured_in_section_end_date=None)
    ArticlePage.objects.live().filter(
        featured_in_homepage_end_date__lte=datetime.now()).update(
            featured_in_homepage=False,
            featured_in_homepage_start_date=None,
            featured_in_homepage_end_date=None)


@task(ignore_result=True)
def promote_articles():
    ArticlePage.objects.live().filter(
        featured_in_latest_start_date__lte=datetime.now()).update(
        featured_in_latest=True)
    ArticlePage.objects.live().filter(
        featured_in_section_start_date__lte=datetime.now()).update(
        featured_in_section=True)
    ArticlePage.objects.live().filter(
        featured_in_homepage_start_date__lte=datetime.now()).update(
        featured_in_homepage=True)


def get_days_section(section=None):
        return [
            section.monday_rotation, section.tuesday_rotation,
            section.wednesday_rotation, section.thursday_rotation,
            section.friday_rotation, section.saturday_rotation,
            section.sunday_rotation]


def get_days_site_settings(site_settings):
        return [
            site_settings.monday_rotation, site_settings.tuesday_rotation,
            site_settings.wednesday_rotation, site_settings.thursday_rotation,
            site_settings.friday_rotation, site_settings.saturday_rotation,
            site_settings.sunday_rotation]


def rotate_latest(main_lang, index, main, site_settings, day):
    """This rotates all the articles that have been marked as
    featured_in_latest. It checks whether current date falls within the set
    date range for content rotation. It then checks whether the current weekday
    is set to rotate, and then rotates an articles for each hour the admin has
    set."""

    def demote_last_featured_article():
        # set the last featured_in_latest article to false
        if main.latest_articles().live().count() >= 2:
            article = main.latest_articles().live().last()
            article.featured_in_latest = False
            article.featured_in_latest_start_date = None
            article.featured_in_latest_end_date = None
            article.save_revision().publish()

    days = get_days_site_settings(site_settings)
    # checks if the current date is within the content rotation range
    if site_settings.content_rotation_start_date and \
            site_settings.content_rotation_end_date:
        if site_settings.content_rotation_start_date < timezone.now() \
                < site_settings.content_rotation_end_date:

            # checks if the current weekday is set to rotate
            if days[day]:
                for time in site_settings.time:
                    time = strptime(str(time), '%H:%M:%S')
                    if time.tm_hour == datetime.now().hour:
                        # get a random article
                        random_article = ArticlePage.objects.live().filter(
                            featured_in_latest=False,
                            languages__language__id=main_lang.id
                        ).descendant_of(index).order_by('?').exact_type(
                            ArticlePage).first()
                        # set random article to feature in latest
                        if random_article:
                            random_article.featured_in_latest_start_date = \
                                datetime.now()
                            random_article.save_revision().publish()
                            promote_articles()
                            demote_last_featured_article()


def rotate_featured_in_homepage(main_lang, day, main):
    def demote_last_featured_article_in_homepage(section):
            articles = ArticlePage.objects.live().filter(
                featured_in_homepage=True,
                languages__language__id=main_lang.id
            ).descendant_of(section).order_by(
                '-featured_in_homepage_start_date')
            if articles.count() >= 2:
                article = articles.last()
                article.featured_in_homepage = False
                article.featured_in_homepage_start_date = None
                article.featured_in_homepage_end_date = None
                article.save_revision().publish()

    for section in SectionPage.objects.descendant_of(main):
        days = get_days_section(section)
        # checks if current date is within the rotation date range
        if section.content_rotation_start_date and \
                section.content_rotation_end_date:
            if section.content_rotation_start_date < timezone.now() \
                    < section.content_rotation_end_date:

                # checks if the current weekday is set to rotate
                if days[day]:
                    for time in section.time:
                        time = strptime(str(time), '%H:%M:%S')
                        if time.tm_hour == datetime.now().hour:
                            random_article = ArticlePage.objects.live().filter(
                                featured_in_homepage=False,
                                languages__language__id=main_lang.id
                            ).descendant_of(section).order_by('?').exact_type(
                                ArticlePage).first()

                            # promotes an article and bumps last one off list
                            if random_article:
                                random_article. \
                                    featured_in_homepage_start_date = \
                                    datetime.now()
                                random_article.save_revision().publish()
                                promote_articles()
                                demote_last_featured_article_in_homepage(
                                    section)


def send_import_email(to_email, context):
    from_email = settings.FROM_EMAIL
    subject = settings.CONTENT_IMPORT_SUBJECT
    body = render_to_string(IMPORT_EMAIL_TEMPLATE, context)
    email_message = EmailMessage(subject, body, from_email, [to_email])
    email_message.send()


def send_validate_email(to_email, context):
    from_email = settings.FROM_EMAIL
    subject = settings.CONTENT_IMPORT_SUBJECT
    body = render_to_string(VALIDATE_EMAIL_TEMPLATE, context)
    email_message = EmailMessage(subject, body, from_email, [to_email])
    email_message.send()


def send_copy_email(to_email, context, csv=None):
    from_email = settings.FROM_EMAIL
    subject = settings.CONTENT_COPY_SUBJECT \
        if hasattr(settings, 'CONTENT_COPY_SUBJECT') else 'Molo Content Copy'
    body = render_to_string(COPY_EMAIL_TEMPLATE, context)
    email_message = EmailMessage(subject, body, from_email, [to_email])
    if csv:
        email_message.attach('file.csv', csv.getvalue(), 'text/csv')
    email_message.send()


def send_copy_failed_email(to_email, context):
    from_email = settings.FROM_EMAIL
    subject = settings.CONTENT_COPY_FAILED_SUBJECT \
        if hasattr(settings, 'CONTENT_COPY_FAILED_SUBJECT') \
        else 'Molo Content Copy Failed'
    body = render_to_string(COPY_FAILED_EMAIL_TEMPLATE, context)
    email_message = EmailMessage(subject, body, from_email, [to_email])
    email_message.send()


@task(ignore_result=True)
def import_content(data, locales, username, email, host):
    repos = api.get_repos(data)
    result = api.validate_content(repos, locales)

    if not result['errors']:
        api.import_content(repos, locales)

    send_import_email(email, {
        'name': username,
        'host': host,
        'errors': result['errors'],
        'warnings': result['warnings']
    })


@task(ignore_result=True)
def validate_content(data, locales, username, email, host):
    repos = api.get_repos(data)
    result = api.validate_content(repos, locales)

    send_import_email(email, {
        'name': username,
        'host': host,
        'type': 'import_failure',
        'errors': result['errors'],
        'warnings': result['warnings']
    })


@task(ignore_result=True)
def molo_consolidated_minute_task():
    """
    Purpose: To reduce the number of db connections, we run all
    tasks in 1 call
    """

    demote_articles()
    promote_articles()
    publish_scheduled_pages()
    clearsessions()


@task(ignore_result=True)
def copy_sections_index(
        section_pk, user_pk, to_pk, copy_revisions, recursive, keep_live):
    section_index = SectionIndexPage.objects.get(pk=section_pk)
    user = User.objects.get(pk=user_pk) if user_pk else None
    to = Page.objects.get(pk=to_pk).specific
    try:
        section_index.copy(
            user=user,
            to=to,
            copy_revisions=copy_revisions,
            recursive=recursive,
            keep_live=keep_live,
            via_celery=True)

        old_main = section_index.get_site().root_page
        new_main = to.get_site().root_page
        create_new_article_relations(old_main, new_main)
        send_copy_email(user.email, {
            'name': (user.get_full_name() or user.username) if user else None,
            'source': section_index.get_parent().title,
            'to': to.title
        })
    except Exception, e:
        logging.error(e, exc_info=True)
        send_copy_failed_email(user.email, {
            'name': (user.get_full_name() or user.username) if user else None,
            'source': section_index.get_parent().title,
            'to': to.title
        })


@task(ignore_result=True)
def import_site(root_url, site_pk, user_pk):
    user = User.objects.get(pk=user_pk) if user_pk else None
    record_keeper = RecordKeeper()
    logger = Logger()
    site = Site.objects.get(pk=site_pk)

    language_importer = LanguageImporter(
        site.pk, root_url,
        record_keeper=record_keeper,
        logger=logger)
    image_importer = ImageImporter(
        site.pk, root_url,
        record_keeper=record_keeper,
        logger=logger)
    content_importer = ContentImporter(
        site.pk, root_url,
        record_keeper=record_keeper,
        logger=logger)

    try:
        # get languages
        language_importer.copy_site_languages()

        image_importer.import_images()

        # copy_content SectionIndexPage
        section_index_page = SectionIndexPage.objects.descendant_of(
            site.root_page).first()
        foreign_section_index_page_id = content_importer.get_foreign_page_id_from_type(  # noqa
            "core.SectionIndexPage")
        content_importer.copy_children(
            foreign_id=foreign_section_index_page_id,
            existing_node=section_index_page)

        # copy_content Banner Pages
        banner_index_page = BannerIndexPage.objects.descendant_of(
            site.root_page).first()
        foreign_banner_index_page_id = content_importer.get_foreign_page_id_from_type(  # noqa
            "core.BannerIndexPage")
        content_importer.copy_children(
            foreign_id=foreign_banner_index_page_id,
            existing_node=banner_index_page)

        # copy_content Footer Pages
        footer_index_page = FooterIndexPage.objects.descendant_of(
            site.root_page).first()
        foreign_footer_index_page_id = content_importer.get_foreign_page_id_from_type(  # noqa
            "core.FooterIndexPage")
        content_importer.copy_children(
            foreign_id=foreign_footer_index_page_id,
            existing_node=footer_index_page)

        # copy_content TagIndexPage
        tag_index_page = TagIndexPage.objects.descendant_of(
            site.root_page).first()
        foreign_tag_index_page_id = content_importer.get_foreign_page_id_from_type(  # noqa
            "core.TagIndexPage")
        content_importer.copy_children(
            foreign_id=foreign_tag_index_page_id,
            existing_node=tag_index_page)

        logger.log(ACTION, "Creating Recommended Articles")
        content_importer.create_recommended_articles()

        logger.log(ACTION, "Creating Related Sections")
        content_importer.create_related_sections()

        logger.log(ACTION, "Creating Nav Tag Relationships")
        content_importer.create_nav_tag_relationships()

        logger.log(ACTION, "Creating Section Tag Relationships")
        content_importer.create_section_tag_relationship()

        logger.log(ACTION, "Creating Banner Page Links")
        content_importer.create_banner_page_links()

        logger.log(ACTION, "Recreating Article Body")
        content_importer.recreate_article_body()

        # create CSV
        foreign_local_map = record_keeper.foreign_local_map["page_map"]

        csvfile = BytesIO()
        writer = csv.writer(csvfile)

        rows = [["foreign_id", "local_id"]]
        for foreign_id, local_id in foreign_local_map.iteritems():
            rows.append([foreign_id, local_id])

        writer.writerows(rows)

        # send email
        send_copy_email(
            user.email,
            {
                'name': ((user.get_full_name() or user.username)
                         if user else None),
                'source': root_url,
                'to': site.root_url,
                'logs': logger.get_email_logs()
            },
            csv=csvfile)
    except Exception, e:
        logging.error(e, exc_info=True)
        send_copy_failed_email(user.email, {
            'name': (user.get_full_name() or user.username) if user else None,
            'source': root_url,
            'to': site.root_url,
            'logs': logger.get_email_logs(),
        })



import requests

from django import forms

from molo.core.api import constants
from molo.core.api.constants import MAIN_IMPORT_FORM_MESSAGES


class SiteImportForm(forms.Form):
    url = forms.CharField(
        required=True,
        max_length=100,
    )

    def clean_url(self):
        url = self.cleaned_data["url"]
        url = url.rstrip("/")

        try:
            requests.get(url + constants.API_PAGES_ENDPOINT)
        except requests.ConnectionError:
            self.add_error(
                "url",
                forms.ValidationError(
                    MAIN_IMPORT_FORM_MESSAGES["connection_error"]
                )
            )
        except requests.RequestException:
            self.add_error(
                "url",
                forms.ValidationError(
                    MAIN_IMPORT_FORM_MESSAGES["bad_request"]
                )
            )
        finally:
            return url


class MainImportForm(SiteImportForm):
    content_type = forms.ChoiceField(
        choices=constants.CONTENT_TYPES,
        required=True,
        widget=forms.RadioSelect
    )


class ArticleImportForm(forms.Form):
    """
    Form to be used for importing articles.
    The list of available articles will be rendered dynamically
    """

    def __init__(self, *args, **kwargs):
        # generate fields dynamically for each article found in the response
        self.importer = kwargs.pop("importer")
        self.parent_id = kwargs.pop("parent")
        super(ArticleImportForm, self).__init__(*args, **kwargs)
        if self.importer and self.importer.content():
            for i, article in enumerate(self.importer.content()):
                self.fields["%s" % i] = forms.BooleanField(
                    label=article["title"]
                )
                self.fields["%s" % i].required = False

    def save(self):
        # soe articles were selected, save them
        selected_choices = [int(k) for k, v in self.cleaned_data.items() if v]
        self.importer.save(
            indexes=selected_choices, parent_id=self.parent_id
        )
        return self.importer


class SectionImportForm(forms.Form):
    """
    Form to be used for importing articles.
    The list of available articles will be rendered dynamically
    """

    def __init__(self, *args, **kwargs):
        # generate fields dynamically for each article found in the response
        self.importer = kwargs.pop("importer")
        self.parent_id = kwargs.pop("parent")
        super(SectionImportForm, self).__init__(*args, **kwargs)
        if self.importer and self.importer.content():
            for item in self.importer.content():
                self.fields["%s" % item["id"]] = forms.BooleanField(
                    label=item["title"]
                )
                self.fields["%s" % item["id"]].required = False

    def save(self):
        selected_choices = [int(k) for k, v in self.cleaned_data.items() if v]
        self.importer.save(
            indexes=selected_choices, parent_id=self.parent_id
        )
        return self.importer



from __future__ import absolute_import, unicode_literals

from django.core.management.base import BaseCommand
from molo.core.models import ArticlePage


class Command(BaseCommand):
    def handle(self, **options):
        ArticlePage.objects.all().update(
            featured_in_latest=False,
            featured_in_latest_start_date=None,
            featured_in_latest_end_date=None,
            featured_in_homepage=False,
            featured_in_homepage_start_date=None,
            featured_in_homepage_end_date=None)



# coding=utf-8
from django.test import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User

from molo.core.models import (
    Main, ReactionQuestionResponse,
    SiteLanguageRelation, Languages, ReactionQuestionChoice)
from molo.core.tests.base import MoloTestCaseMixin


class TestReactionQuestionResultsAdminView(TestCase, MoloTestCaseMixin):
    def setUp(self):
        self.mk_main()
        main = Main.objects.all().first()
        self.english = SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(main.get_site()),
            locale='en',
            is_active=True)

        self.french = SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(main.get_site()),
            locale='fr',
            is_active=True)

        self.yourmind = self.mk_section(
            self.section_index, title='Your mind')

        self.yourmind_fr = self.mk_section_translation(
            self.yourmind, self.french, title='Your mind in french')

        self.mk_main2()
        self.main2 = Main.objects.all().last()
        self.language_setting2 = Languages.objects.create(
            site_id=self.main2.get_site().pk)
        self.english2 = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting2,
            locale='en',
            is_active=True)

        self.spanish = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting2,
            locale='es',
            is_active=True)

        self.yourmind2 = self.mk_section(
            self.section_index2, title='Your mind2')

    def test_reaction_question_appears_in_wagtail_admin(self):
        User.objects.create_superuser(
            username='testuser', password='password', email='test@email.com')
        self.client.login(username='testuser', password='password')
        article = self.mk_article(self.yourmind)
        article_site_2 = self.mk_article(self.yourmind2)
        question = self.mk_reaction_question(self.reaction_index, article)
        question2 = self.mk_reaction_question(
            self.reaction_index2, article_site_2)
        response = self.client.get(
            '/admin/core/reactionquestion/'
        )
        self.assertContains(
            response,
            '<a href="/admin/reactionquestion/%s/results/">'
            'Test Question</a>' % question.pk)
        self.assertNotContains(
            response,
            '<a href="/admin/reactionquestion/%s/results/">'
            'Test Question</a>' % question2.pk)

    def test_article_appears_in_wagtail_admin_summary(self):
        User.objects.create_superuser(
            username='testuser', password='password', email='test@email.com')
        self.client.login(username='testuser', password='password')
        article = self.mk_article(self.yourmind)
        article_site_2 = self.mk_article(self.yourmind2)
        self.mk_reaction_question(self.reaction_index, article)
        self.mk_reaction_question(
            self.reaction_index2, article_site_2)
        response = self.client.get(
            '/admin/core/articlepage/'
        )
        self.assertContains(
            response,
            '<a href="/admin/reactionquestion/%s/results/summary/">'
            'Test page 0</a>' % article.pk)
        self.assertNotContains(
            response,
            '<a href="/admin/reactionquestion/%s/results/summary/">'
            'Test page 0</a>' % article_site_2.pk)

    def test_reaction_question_results_view(self):
        super_user = User.objects.create_superuser(
            username='testuser', password='password', email='test@email.com')
        self.client.login(username='testuser', password='password')

        article = self.mk_article(self.yourmind)
        question = self.mk_reaction_question(self.reaction_index, article)
        choice1 = question.get_children().first()
        response = self.client.post(reverse(
            'reaction-vote', kwargs={
                'question_id': question.id, 'article_slug': article.slug}),
            {'choice': choice1.id})

        response = self.client.get(
            '/admin/reactionquestion/{0}/results/'.format(question.id)
        )

        expected_headings_html = '<tr><th>Submission Date</th><th>Answer</th>'\
                                 '<th>User</th><th>Article</th></tr>'

        self.assertContains(response, expected_headings_html, html=True)
        self.assertContains(response, choice1.title)
        self.assertContains(response, super_user.username)
        self.assertContains(response, article.title)

        # test CSV download
        response = self.client.get(
            '/admin/reactionquestion/{0}/results/?action=download'.format(
                question.id)
        )
        created_date = ReactionQuestionResponse.objects.first().created_at

        expected_output = (
            'Submission Date,Answer,User,Article\r\n'
            '{0},yes,{1},{2}\r\n'
        ).format(
            created_date,
            super_user.username, article.title
        )
        self.assertContains(response, expected_output)

    def test_reaction_question_results_summary_view(self):
        User.objects.create_superuser(
            username='testuser', password='password', email='test@email.com')
        self.client.login(username='testuser', password='password')

        article = self.mk_article(self.yourmind)
        question = self.mk_reaction_question(self.reaction_index, article)
        choice1 = question.get_children().first()
        response = self.client.post(reverse(
            'reaction-vote', kwargs={
                'question_id': question.id, 'article_slug': article.slug}),
            {'choice': choice1.id})

        response = self.client.get(
            '/admin/reactionquestion/{0}/results/summary/'.format(article.id)
        )

        expected_headings_html = '<tr><th>Article</th><th>yes</th>'\
                                 '<th>maybe</th><th>no</th></tr>'
        self.assertContains(response, expected_headings_html, html=True)
        self.assertContains(response, 'Test page 0')

        # test CSV download
        response = self.client.get(
            '/admin/reactionquestion/{0}/'
            'results/summary/?action=download'.format(article.id)
        )

        expected_output = (
            'Article,yes,maybe,no\r\n'
            '{0},1,0,0\r\n'
        ).format(
            article.title,
        )
        self.assertContains(response, expected_output)


class TestReactionQuestions(TestCase, MoloTestCaseMixin):

    def setUp(self):
        self.mk_main()
        main = Main.objects.all().first()
        self.english = SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(main.get_site()),
            locale='en',
            is_active=True)

        self.french = SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(main.get_site()),
            locale='fr',
            is_active=True)

        self.yourmind = self.mk_section(
            self.section_index, title='Your mind')

        self.yourmind_fr = self.mk_section_translation(
            self.yourmind, self.french, title='Your mind in french')

        self.mk_main2()
        self.main2 = Main.objects.all().last()
        self.language_setting2 = Languages.objects.create(
            site_id=self.main2.get_site().pk)
        self.english2 = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting2,
            locale='en',
            is_active=True)

        self.spanish = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting2,
            locale='es',
            is_active=True)

        self.yourmind2 = self.mk_section(
            self.section_index2, title='Your mind2')

    def test_can_react_on_article(self):
        article = self.mk_article(self.yourmind)
        question = self.mk_reaction_question(self.reaction_index, article)
        self.user = self.login()
        response = self.client.get(article.url)
        self.assertContains(response, question.title)
        for choice in question.get_children():
            self.assertContains(response, choice.title)
        choice1 = question.get_children().first()
        self.assertEquals(ReactionQuestionResponse.objects.all().count(), 0)
        response = self.client.post(reverse(
            'reaction-vote', kwargs={
                'question_id': question.id, 'article_slug': article.slug}),
            {'choice': choice1.id})
        self.assertEquals(ReactionQuestionResponse.objects.all().count(), 1)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(
            response['Location'], '/reaction/test-page-0/20/yes/feedback/')
        response = self.client.get('/reaction/test-page-0/20/yes/feedback/')
        self.assertContains(
            response, '<a href="/sections-main-1/your-mind/test-page-0/">')
        self.assertContains(response, 'well done')
        # test user can only submit once
        response = self.client.post(reverse(
            'reaction-vote', kwargs={
                'question_id': question.id, 'article_slug': article.slug}),
            {'choice': choice1.id})
        self.assertEquals(ReactionQuestionResponse.objects.all().count(), 1)

    def test_correct_reaction_shown_for_locale(self):
        article = self.mk_article(self.yourmind)
        question = self.mk_reaction_question(self.reaction_index, article)
        self.mk_article_translation(
            article,
            self.french,
            title=article.title + ' in french',)
        translated_question = self.mk_reaction_translation(
            question,
            article,
            self.french,
            title=question.title + ' in french',)
        translated_choice = ReactionQuestionChoice(
            title='ja', success_message='mooi gedoen')
        question.add_child(instance=translated_choice)
        translated_choice.save_revision().publish()
        translated_choice = self.mk_translation(
            question.get_first_child(), self.french, translated_choice)

        self.client.get('/locale/fr/')
        response = self.client.get(
            '/sections-main-1/your-mind/test-page-0-in-french/')
        self.assertContains(response, 'Test Question in french')
        response = self.client.post(reverse(
            'reaction-vote', kwargs={
                'question_id': translated_question.id,
                'article_slug': article.slug}),
            {'choice': translated_choice.id})
        self.assertEquals(ReactionQuestionResponse.objects.all().count(), 1)
        response = self.client.get('/reaction/test-page-0/20/yes/feedback/')
        self.assertContains(
            response, '<a href="/sections-main-1/your-mind/test-page-0/">')
        self.assertContains(response, 'mooi gedoen')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0028_rename_homepage_to_bannerpage'),
    ]

    operations = [
        migrations.AddField(
            model_name='articlepage',
            name='uuid',
            field=models.CharField(max_length=32, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='sectionpage',
            name='uuid',
            field=models.CharField(max_length=32, null=True, blank=True),
        ),
    ]



import json

from babel import Locale
from babel.core import UnknownLocaleError

from django.db.models.base import ValidationError

from unicore.content.models import Category, Page

from molo.core.models import (
    PageTranslation, SectionPage, ArticlePage, FooterPage,
    SectionIndexPage, FooterIndexPage, SiteLanguageRelation, Languages, Main)
from molo.core.content_import.helpers.get_image import get_image_file
from molo.core.content_import.helpers.locales import filter_locales_in_repo
from molo.core.content_import.utils import hash
from molo.core.utils import generate_slug


def import_repo(repo, main_locale, children, should_nest=False):
    children = filter_locales_in_repo(repo, children)

    for locale in [main_locale] + children:
        lang = create_language(repo, locale, locale == main_locale)
        index = get_or_create_index(repo, lang, should_nest)
        stray_index = get_or_create_stray_index(repo, lang, False)
        import_locale_content(repo, lang, index, stray_index)
        update_pages_with_linked_page_field(repo, locale)


def get_models(repo, cls, **kw):
    qs = repo.workspace.S(cls)
    qs = qs.filter(**kw)

    # S() only returns 10 results if you don't ask for more
    return qs.order_by('position').everything()


def get_sections_index():
    return SectionIndexPage.objects.live().first()


def get_footer_index():
    return FooterIndexPage.objects.live().first()


def get_or_create_index(repo, lang, should_nest):
    if should_nest:
        return repo_section_for_language(repo, lang)
    else:
        return get_sections_index()


def get_or_create_stray_index(repo, lang, should_nest):
    if should_nest:
        return get_or_create_index(repo, lang, should_nest)
    else:
        return get_footer_index()


def create_language(repo, locale, is_main):
    main = Main.objects.all().first()
    language_setting, _ = Languages.objects.get_or_create(
        site_id=main.get_site().pk)
    try:
        language, _ = SiteLanguageRelation.objects.get_or_create(
            language_setting=language_setting,
            locale=Locale.parse(locale).language,
            is_main_language=is_main,
            is_active=True)
    except UnknownLocaleError:
        language, _ = SiteLanguageRelation.objects.get_or_create(
            language_setting=language_setting,
            locale=locale.replace('_', '-'),
            is_main_language=False,
            is_active=True)

    return {
        'locale': locale,
        'language': language,
    }


def import_locale_content(repo, lang, index, stray_index):
    import_all_categories(repo, lang, index)
    import_all_pages(repo, lang, stray_index)


def repo_section_datum(repo, lang):
    # NOTE For sections that correspond to a repository (for the multi-repo
    # import case), We can't simply rely on the ORM to create a uuid for us,
    # since we need to retrieve the section if it already exists instead of
    # creating it again. This means we need to find the section by its
    # corresponding repository's name and the locale that the section was
    # created for. At the moment, we use the uuid field for this, though isn't
    # quite right -- uuid's need to be universally unique. A more robust
    # solution might be to support a metadata field on the SectionPage model
    # and query by this field for this case

    # NOTE At the moment, we are using the repository title as the title for
    # the corresponding section we create, regardless of the language the
    # section is created for. We need a way of creating a title from the
    # repository in the given language
    return {
        'uuid': hash((repo.name, lang['locale'])),
        'title': repo.title
    }


def repo_section_for_language(repo, lang):
    datum = repo_section_datum(repo, lang)
    index = get_sections_index()

    if is_main_language(lang):
        return get_or_create(SectionPage, index, **datum)
    else:
        return get_or_create_translation(SectionPage, lang, index, **datum)


def import_categories_for_child_language(repo, category, lang, index):
    if category.source:
        try:
            main_lang_page = SectionPage.objects.get(uuid=category.source)
            translated_section = import_section_content(
                repo, category, lang, index)
            PageTranslation.objects.get_or_create(
                page=main_lang_page,
                translated_page=translated_section)
        except SectionPage.DoesNotExist:
            print "couldn't find", category.source, (
                SectionPage.objects.all().values('uuid'))
    else:
        print "no source found for: ", category.source, (
            SectionPage.objects.all().values('uuid'))


def import_all_categories(repo, lang, index):
    category_qs = get_models(repo, Category, language=lang['locale'])

    if lang['language'].is_main_language:
        import_fn = import_section_content
    else:
        import_fn = import_categories_for_child_language

    for c in category_qs:
        import_fn(repo, c, lang, index)


def import_all_pages(repo, lang, stray_index):
    for p in get_models(repo, Page, language=lang['locale']):
        import_page_content(repo, p, lang, stray_index)


def is_main_language(lang):
    return lang['language'].is_main_language


def get_or_create_from_model(cls, obj, parent):
    return get_or_create(cls, parent, obj.uuid, obj.title)


def get_or_create_translation_from_model(cls, obj, lang, parent):
    return get_or_create_translation(cls, lang, parent, obj.uuid, obj.title)


def get_or_create(cls, parent, uuid, title):
    if cls.objects.filter(uuid=uuid).exists():
        return cls.objects.get(uuid=uuid)

    instance = cls(uuid=uuid, title=title)

    try:
        parent.add_child(instance=instance)
    except ValidationError:
        # non-ascii titles need slugs to be manually generated
        instance.slug = generate_slug(title)
        parent.add_child(instance=instance)

    instance.save_revision().publish()
    return instance


def get_or_create_translation(cls, lang, parent, uuid, title):
    instance = get_or_create(cls, parent, uuid, title)

    language_relation = instance.languages.first()
    language_relation.language = lang['language']
    language_relation.save()

    return instance


def import_section_content(repo, c, lang, parent):
    if is_main_language(lang):
        section = get_or_create_from_model(SectionPage, c, parent)
    else:
        section = get_or_create_translation_from_model(
            SectionPage, c, lang, parent)

    section.description = c.subtitle
    if c.image_host and c.image:
        section.image = get_image_file(c.image_host, c.image)

    section.save_revision().publish()

    return section


def import_page_content(repo, p, lang, stray_index):
    if is_main_language(lang):
        if p.primary_category:
            try:
                section = SectionPage.objects.get(uuid=p.primary_category)
                page = get_or_create_from_model(ArticlePage, p, section)
            except SectionPage.DoesNotExist:
                print "couldn't find primary category ", \
                    p.primary_category, \
                    SectionPage.objects.all().values('uuid')
                return None
        else:
            # special case for articles with no primary category
            # this assumption is probably wrong..
            # but we have no where else to put them
            page = get_or_create_from_model(FooterPage, p, stray_index)
    else:
        try:
            main_instance = ArticlePage.objects.get(uuid=p.source).specific

            page = get_or_create_translation_from_model(
                main_instance.__class__, p, lang, main_instance.get_parent())

            PageTranslation.objects.get_or_create(
                page=main_instance,
                translated_page=page)
        except ArticlePage.DoesNotExist:
            print "No source found for: ", p.source, (
                ArticlePage.objects.all().values('uuid'))
            return None

    page.subtitle = p.description
    page.body = json.dumps([{
        'type': 'paragraph',
        'value': p.content
    }])

    is_featured = p.featured if p.featured else False
    is_featured_in_category = p.featured_in_category \
        if p.featured_in_category else False

    page.featured_in_latest = is_featured
    page.featured_in_homepage = is_featured_in_category
    for tag in p.author_tags:
        page.metadata_tags.add(tag)
    if p.image_host and p.image:
        page.image = get_image_file(p.image_host, p.image)

    page.save_revision().publish()

    return page


def update_pages_with_linked_page_field(repo, locale):
    for p in repo.workspace.S(Page).filter(language=locale).everything():
        if p.linked_pages:
            for lp in p.linked_pages:
                try:
                    page = ArticlePage.objects.get(uuid=p.uuid)
                    page.body.stream_data.append(
                        {u'type': u'page',
                         u'value': ArticlePage.objects.get(uuid=lp).pk})
                    page.save_revision().publish()
                except ArticlePage.DoesNotExist:
                    print 'Linked page does not exist %s' % lp



from daterange_filter.filter import DateRangeFilter
from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.template.defaultfilters import truncatechars
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from wagtail.contrib.modeladmin.options import (
    ModelAdmin as WagtailModelAdmin,
)
from wagtail.contrib.modeladmin.options import ModelAdminGroup

from molo.core.models import (
    ReactionQuestion, ReactionQuestionResponse, ArticlePage,
    ArticlePageLanguageProxy, SectionPage
)


class DateFilter(DateRangeFilter):
    template = 'admin/date_range_filter.html'


class ReactionQuestionAdmin(admin.ModelAdmin):
    list_display = ('title', 'live')
    fieldsets = (
        (
            None,
            {'fields': ('title', )}
        ),
    )
    readonly_fields = ['title']


class ReactionQuestionResponseAdmin(admin.ModelAdmin):
    list_display = ('question', 'choice', 'user', 'article')
    fieldsets = (
        (
            None,
            {'fields': ('question', 'choice', 'user', 'article')}
        ),
    )
    readonly_fields = ['question', 'choice']


admin.site.register(ReactionQuestion, ReactionQuestionAdmin)
admin.site.register(ReactionQuestionResponse, ReactionQuestionResponseAdmin)


class ReactionQuestionsModelAdmin(WagtailModelAdmin, ReactionQuestionAdmin):
    model = ReactionQuestion
    menu_label = 'Reaction Question'
    menu_icon = 'doc-full'
    add_to_settings_menu = False
    list_display = ('responses', 'live')

    def responses(self, obj, *args, **kwargs):
        url = reverse('reaction-question-results-admin', args=(obj.id,))
        return '<a href="%s">%s</a>' % (url, obj)

    def get_queryset(self, request):
        qs = super(ReactionQuestionAdmin, self).get_queryset(request)
        # Only show questions related to that site
        main = request.site.root_page
        return qs.descendant_of(main)

    responses.allow_tags = True
    responses.short_description = 'Title'


class ReactionQuestionsSummaryModelAdmin(
        WagtailModelAdmin, ReactionQuestionAdmin):
    model = ArticlePage
    menu_label = 'Reaction Question Summary'
    menu_icon = 'doc-full'
    add_to_settings_menu = False
    list_display = ('articles', 'live')

    def articles(self, obj, *args, **kwargs):
        url = reverse(
            'reaction-question-article-results-admin', args=(obj.id,))
        return '<a href="%s">%s</a>' % (url, obj)

    def get_queryset(self, request):
        qs = ArticlePage.objects.descendant_of(
            request.site.root_page).filter(
                languages__language__is_main_language=True).exclude(
            reaction_questions=None
        )
        return qs
    articles.allow_tags = True
    articles.short_description = 'Title'


class ArticleAdmin(admin.ModelAdmin):
    list_display = (
        'title', 'live', 'first_published_at', 'owner',
        'latest_revision_created_at', 'go_live_at'
        'featured_in_latest', 'featured_in_homepage', 'featured_in_section'
    )

    fieldsets = (
        (
            None,
            {'fields': ('title', )}
        ),
    )
    readonly_fields = ['title']


class StatusCategoryListFilter(admin.SimpleListFilter):
    title = _('Status')
    parameter_name = 'status'

    def lookups(self, request, model_admin):
        return (
            ('published', 'Published'),
            ('in_review', 'In Review'),
            ('draft', 'Draft')
        )

    def queryset(self, request, queryset):
        if self.value() == 'in_review':
            return queryset.filter(revisions__submitted_for_moderation=True)
        elif self.value() == 'published':
            return queryset.filter(live=True)
        elif self.value() == 'draft':
            return queryset.filter(
                ~Q(revisions__submitted_for_moderation=True) & ~Q(live=True)
            )
        else:
            return queryset


class SectionListFilter(admin.SimpleListFilter):
    title = _('Sections')
    parameter_name = 'section'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        list_tuple = []
        for article in ArticlePage.objects.all():
            if article.get_parent_section():
                section_tuple = (
                    article.get_parent_section().id,
                    article.get_parent_section().title
                )
                if section_tuple not in list_tuple:
                    list_tuple.append(section_tuple)
        return list_tuple

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        if self.value():
            try:
                section = SectionPage.objects.get(id=self.value())
                return queryset.child_of(section).all()
            except (ObjectDoesNotExist, MultipleObjectsReturned):
                return None


class ArticleModelAdmin(WagtailModelAdmin, ArticleAdmin):

    model = ArticlePageLanguageProxy
    menu_label = 'Articles'
    menu_icon = 'doc-full-inverse'
    list_display = [
        'article_title', 'section', 'live', 'status',
        'first_published_at', 'owner', 'first_created_at',
        'latest_revision_created_at', 'last_edited_by',
        'image_img', 'tags_html',
        'featured_in_latest', 'featured_in_homepage', 'featured_in_section',
    ]
    list_filter = [
        StatusCategoryListFilter,
        SectionListFilter,
        ('first_published_at', DateFilter),
        ('latest_revision_created_at', DateFilter),
        'featured_in_latest',
        'featured_in_homepage',
        'featured_in_section',
        'tags'
    ]
    search_fields = ('title', 'subtitle')

    def article_title(self, obj):
        return obj.title
    article_title.admin_order_field = 'title'
    article_title.short_description = 'Article Title'

    def image_img(self, obj):
        if obj.image:

            return u'<img src="%s" />' % (
                obj.image.get_rendition('height-50').url
            )

    image_img.short_description = 'Image'
    image_img.allow_tags = True

    def section(self, obj):
        if obj.get_parent_section():
            return format_html(
                "<a href='{section}'>{section_title}</a>",
                section_title=obj.get_parent_section(
                ).get_admin_display_title(),
                section=reverse(
                    'wagtailadmin_explore',
                    args=[obj.get_parent_section().id]
                )
            )

    section.short_description = 'Section'
    section.allow_tags = True

    def tags_html(self, obj):
        if obj.tags_list():
            display_value = ", ".join(obj.tags_list())
            result = format_html(
                '<a href="{}">{}</a>'.format(
                    reverse("wagtailadmin_pages:edit", args=[obj.id]),
                    truncatechars(display_value.upper(), 30)
                )
            )
            return result

    tags_html.short_description = 'Tags'
    tags_html.allow_tags = True

    def status(self, obj):
        if obj.get_latest_revision().submitted_for_moderation:
            return "In Review"
        elif obj.live:
            return "Published"
        else:
            return "Draft"

    status.short_description = 'Status'
    status.allow_tags = True

    def last_edited_by(self, obj):
        if obj.get_latest_revision():
            return obj.get_latest_revision().user

    last_edited_by.short_description = 'Last edited by'
    last_edited_by.allow_tags = True

    def first_created_at(self, obj):
        if obj.revisions.first():
            return obj.revisions.first().created_at

    first_created_at.short_description = 'Created at'
    first_created_at.allow_tags = True

    def get_queryset(self, request):
        qs = ArticlePageLanguageProxy.objects.descendant_of(
            request.site.root_page)
        return qs


class AdminViewGroup(ModelAdminGroup):
    menu_label = 'Admin View'
    menu_icon = 'folder-open-inverse'
    menu_order = 1500
    items = (
        ArticleModelAdmin,
    )



"""
Various importers for the different content types
"""
import math
import json
import requests
import logging
from io import BytesIO

from django.core.files.images import ImageFile

from wagtail.wagtailcore.models import Page
from wagtail.wagtailimages.models import Image

from molo.core.models import *  # noqa
from molo.core.api.constants import (
    API_IMAGES_ENDPOINT,
    API_PAGES_ENDPOINT,
    ACTION,
    SUCCESS,
    ERROR,
    WARNING,
)
from molo.core.api.errors import *  # noqa
from molo.core.utils import separate_fields


# functions used to find images
def get_image_attributes(base_url, image_id):
    image_attrs = requests.get(
        base_url + API_IMAGES_ENDPOINT + str(image_id)
    ).json()
    return image_attrs


def get_image(base_url, image_id):
    # TODO: guard against non-existent images
    image_attributes = get_image_attributes(base_url, image_id)
    image_file = requests.get(image_attributes["file"])
    image = Image(
        title=image_attributes["title"],
        file=ImageFile(
            BytesIO(image_file.content), name=image_attributes["title"]
        )
    )
    image.save()
    return image


def list_of_objects_from_api(url):
    '''
    API only serves 20 pages by default
    This fetches info on all of items and return them as a list

    Assumption: limit of API is not less than 20
    '''
    response = requests.get(url)

    content = json.loads(response.content)
    count = content["meta"]["total_count"]

    if count <= 20:
        return content["items"]
    else:
        items = [] + content["items"]
        num_requests = int(math.ceil(count // 20))

        for i in range(1, num_requests + 1):
            paginated_url = "{}?limit=20&offset={}".format(
                url, str(i * 20))
            paginated_response = requests.get(paginated_url)
            items = items + json.loads(paginated_response.content)["items"]
    return items


class PageImporter(object):

    def __init__(self, base_url=None, content=None, content_type=None):
        self._content_type = content_type
        self._fields = []
        self._content = content
        self._base_url = base_url

    def get_content_from_url(self, base_url):
        """
        Sections can have SectionPage and ArticlePage child objects.
        These have different fields, and thus have to be treated
        differently.
        """
        # assemble url
        base_url = base_url.rstrip("/")
        url = base_url + API_PAGES_ENDPOINT + "?type=" + self._content_type + \
            "&fields=" + ",".join(self._fields) + \
            "&order=latest_revision_created_at"

        # make request
        try:
            response = requests.get(url)
            self._base_url = base_url
            self._content = response.json()
            self._content = self._content["items"]
            return self._content
        except requests.exceptions.ConnectionError:
            return "No content could be found from {}. " \
                "Are you sure this is the correct URL?".format(base_url)
        except requests.exceptions.RequestException:
            return "Content could not be imported at this time. " \
                   "Please try again later."

    def content(self):
        if self._content:
            return self._content
        return []

    def save(self, indexes, parent_id):
        pass


class ArticlePageImporter(PageImporter):

    def __init__(self, base_url=None, content=None, content_type=None):
        super(ArticlePageImporter, self).__init__(
            base_url=base_url, content=content, content_type=content_type
        )
        self._content_type = "core.ArticlePage"
        self._fields = ArticlePage.get_api_fields()

    def save(self, indexes, parent_id):
        if self.content():
            parent = Page.objects.get(id=parent_id)
            for index in indexes:
                # Remove "id" and "meta" fields
                selected_article = self.content()[index]
                fields, nested_fields = separate_fields(selected_article)
                article = ArticlePage(**fields)

                # TODO: u'related_sections'
                # process the nested fields
                if ("tags" in nested_fields) and nested_fields["tags"]:
                    article.tags.add(", ".join(nested_fields["tags"]))

                if ("metadata_tags" in nested_fields) and \
                        nested_fields["metadata_tags"]:
                    article.metadata_tags.add(
                        ", ".join(nested_fields["metadata_tags"])
                    )

                if ("body" in nested_fields) and nested_fields["body"]:
                    article.body = json.dumps(nested_fields["body"])

                if ("image" in nested_fields) and nested_fields["image"]:
                    article.image = get_image(
                        self._base_url, nested_fields["image"]["id"]
                    )

                parent.add_child(instance=article)
                parent.save_revision().publish()


class SectionPageImporter(PageImporter):

    def __init__(self, base_url=None, content=None, content_type=None):
        super(SectionPageImporter, self).__init__(
            base_url=base_url, content=content, content_type=content_type
        )
        self._content_type = "core.SectionPage"
        self._fields = SectionPage.get_api_fields()

    def _save_item(self, item, parent):
        if item["meta"]["type"] == "core.SectionPage":
            flat_fields, nested_fields = separate_fields(item)
            child_section = SectionPage(**flat_fields)

            if ("image" in nested_fields) and nested_fields["image"]:
                child_section.image = get_image(
                    self._base_url, nested_fields["image"]["id"]
                )

            parent.add_child(instance=child_section)
            parent.save_revision().publish()
            return child_section

        elif item["meta"]["type"] == "core.ArticlePage":
            flat_fields, nested_fields = separate_fields(item)
            child_article = ArticlePage(**flat_fields)
            if ("tags" in nested_fields) and nested_fields["tags"]:
                child_article.tags.add(
                    ", ".join(nested_fields["tags"])
                )

            if ("metadata_tags" in nested_fields) and \
                    nested_fields["metadata_tags"]:
                child_article.metadata_tags.add(
                    ", ".join(nested_fields["metadata_tags"])
                )

            if ("body" in nested_fields) and nested_fields["body"]:
                child_article.body = json.dumps(nested_fields["body"])

            if ("image" in nested_fields) and nested_fields["image"]:
                child_article.image = get_image(
                    self._base_url, nested_fields["image"]["id"]
                )

            parent.add_child(instance=child_article)
            parent.save_revision().publish()
            return child_article

    def process_child_section(self, id, parent):
        response = requests.get(
            self._base_url + API_PAGES_ENDPOINT + str(id)
        ).json()
        ancestor = self._save_item(response, parent)
        if response["meta"]["children"]:
            for child_id in response["meta"]["children"]["items"]:
                self.process_child_section(child_id, ancestor)
        return ancestor

    def recursive_children(self, node):
        results = [node['id']]
        if len(node['children']) > 0:
            for child in node['children']:
                results.extend(self.recursive_children(child))
        return results

    def save(self, indexes, parent_id):
        """
        Save the selected section. This will save the selected section
        as well as its direct child pages obtained through the ?child_of
        query parameter. The ?descendant_of query parameter is probably
         better suited because it all pages under that part of the tree will
         be obtained. The problem , however, is that that will require being
         able to traverse the tree and recreate parent-child relationships
         after they are imported
        """
        if self.content():
            parent = Page.objects.get(id=parent_id)

            # Save the selected section page
            response = requests.get(
                self._base_url + API_PAGES_ENDPOINT + str(indexes[0]) + "/"
            )

            section_page = response.json()
            self.process_child_section(section_page["id"], parent)


class RecordKeeper(object):
    def __init__(self):
        # maps foreign IDs to local IDs
        # used when a new item is created
        self.foreign_local_map = {
            "page_map": {},
            "image_map": {},
        }

        # maps local id to list of foreign page ids
        # used when a foreign item is referenced
        self.foreign_to_many_foreign_map = {
            "recommended_articles": {},
            "related_sections": {},
            "nav_tags": {},
            "section_tags": {},
            "reaction_questions": {},
        }

        # maps local page to foreign id
        self.foreign_to_foreign_map = {
            "banner_link_page": {}
        }

        # maps foreign ID to article body blob
        self.article_bodies = {}

    def record_relation(self, id_map_key, foreign_page_id, local_page_id):
        record = self.foreign_local_map[id_map_key]
        if (foreign_page_id in record and
                record[foreign_page_id] != local_page_id):
            raise RecordOverwriteError("RecordOverwriteError", None)
        else:
            record[foreign_page_id] = local_page_id

    def get_local(self, id_map_key, foreign_page_id):
        record = self.foreign_local_map[id_map_key]
        if foreign_page_id in record:
            return record[foreign_page_id]
        else:
            raise ReferenceUnimportedContent(
                "Unimported content foreign ID: {}".format(foreign_page_id),
                None)

    def record_foreign_relations(self, field, related_item_key, id_map_key,
                                 nested_fields, page_id, logger=None):
        record_keeper = self.foreign_to_many_foreign_map[id_map_key]
        if ((field in nested_fields) and nested_fields[field]):
            relationship_object_list = nested_fields[field]

            # Assumption: this item is only processed once
            # TODO: create override checks
            record_keeper[page_id] = []

            for relationship_object in relationship_object_list:
                try:
                    if related_item_key in relationship_object:
                        related_item = relationship_object[related_item_key]
                        if not related_item:
                            continue
                        if "id" in related_item:
                            foreign_id = related_item["id"]
                            record_keeper[page_id].append(foreign_id)
                        else:
                            raise ImportedContentInvalid(
                                ("key of 'id' does not exist in related_item"
                                 " of type: {}").format(related_item_key),
                                None)
                    else:
                        raise ImportedContentInvalid(
                            ("key of '{}' does not exist in nested_field"
                             " of type: {}").format(related_item_key, field),
                            None)
                except ImportedContentInvalid as e:
                    raise
                except Exception as e:
                    if logger:
                        logger.log(
                            ERROR,
                            "record_foreign_relations error",
                            {
                                "exception": e,
                                "field": field,
                                "related_item_key": related_item_key,
                                "id_map_key": id_map_key,
                                "foreign_page_id": page_id,
                            })
                    continue

    def record_page_relation(self, foreign_page_id, local_page_id):
        self.record_relation("page_map", foreign_page_id, local_page_id)

    def record_image_relation(self, foreign_page_id, local_page_id):
        self.record_relation("image_map", foreign_page_id, local_page_id)

    def get_local_page(self, foreign_page_id):
        return self.get_local("page_map", foreign_page_id)

    def get_local_image(self, foreign_page_id):
        return self.get_local("image_map", foreign_page_id)

    def record_recommended_articles(self, nested_fields, page_id, logger=None):
        self.record_foreign_relations(
            "recommended_articles", "recommended_article",
            "recommended_articles", nested_fields, page_id, logger=logger)

    def record_related_sections(self, nested_fields, page_id, logger=None):
        self.record_foreign_relations(
            "related_sections", "section",
            "related_sections", nested_fields, page_id, logger=logger)

    def record_section_tags(self, nested_fields, page_id, logger=None):
        self.record_foreign_relations(
            "section_tags", "tag",
            "section_tags", nested_fields, page_id, logger=logger)

    def record_nav_tags(self, nested_fields, page_id, logger=None):
        self.record_foreign_relations(
            "nav_tags", "tag",
            "nav_tags", nested_fields, page_id, logger=logger)

    def record_reaction_questions(self, nested_fields, page_id, logger=None):
        self.record_foreign_relations(
            "reaction_questions", "reaction_question",
            "reaction_questions", nested_fields, page_id, logger=logger)

    def record_banner_page_link(self, nested_fields, page_id, logger=None):
        field = "banner_link_page"
        id_map_key = "banner_link_page"
        record_keeper = self.foreign_to_foreign_map[id_map_key]
        if ((field in nested_fields) and nested_fields[field]):
            relationship_object = nested_fields[field]
            record_keeper[page_id] = relationship_object["id"]


class BaseImporter(object):
    def __init__(self, site_pk, base_url, record_keeper=None, logger=None):
        # TODO: handle case where base_url is not valid
        self.base_url = self.format_base_url(base_url)
        self.api_url = '{}/api/v2/'.format(self.base_url)
        self.site_pk = site_pk
        self.language_setting, created = Languages.objects.get_or_create(
            site_id=self.site_pk)
        self.record_keeper = record_keeper
        self.logger = logger

    def format_base_url(self, base_url):
        if base_url[-1] == '/':
            return base_url[:-1]
        else:
            return base_url

    def log(self, log_type, message, context=None, depth=0):
        if self.logger:
            self.logger.log(log_type, message, context, depth)


class ImageImporter(BaseImporter):
    def __init__(self, site_pk, base_url, record_keeper=None, logger=None):
        super(ImageImporter, self).__init__(site_pk, base_url,
                                            record_keeper=record_keeper,
                                            logger=logger)
        self.image_url = "{}images/".format(self.api_url)
        self.image_hashes = {}
        self.get_image_details()

    def get_image_details(self):
        '''
        Create a reference of site images by hash

        If there are duplicate images, only store the first
        and create warnings for other images
        '''
        if Image.objects.count() == 0:
            return None

        total = Image.objects.count()
        count = 1
        for local_image in Image.objects.all():
            if not hasattr(local_image, 'image_info'):
                ImageInfo.objects.create(image=local_image)
                local_image.refresh_from_db()

            hash_ = local_image.image_info.image_hash

            if hash_ in self.image_hashes:
                self.log(WARNING, "Image found with matching hash", context={
                    "composite hash": hash_,
                    "hashed image ID": self.image_hashes[hash_].id,
                    "matching image ID": local_image.id,
                })
            else:
                self.image_hashes[hash_] = local_image

            self.log(ACTION, "{}/{} images processed".format(count, total))
            count += 1

    def get_replica_image(self, img_hash):
        if img_hash in self.image_hashes:
            return self.image_hashes[img_hash]
        return None

    def fetch_and_create_image(self, url, image_title):
        '''
        fetches, creates image object

        returns tuple with Image object and context dictionary containing
        request URL
        '''

        context = {
            "file_url": url,
            "foreign_title": image_title,
        }
        try:
            image_file = requests.get(url)
            local_image = Image(
                title=image_title,
                file=ImageFile(
                    BytesIO(image_file.content),
                    name=image_title
                )
            )
            local_image.save()
            return (local_image, context)
        except Exception as e:
            context.update({
                "exception": e,
            })
            raise ImageCreationFailed(context, None)

    def import_image(self, image_id):
        '''
        Imports and returns tuple with image and context dict

        Input: foreign image ID

        Output: (Image: imported image, Dict: info about import)

        Side effects: If Importer object has a record_keeper, it
        will update the record of foreign to local images.

        Attempts to avoid duplicates by matching image dimensions
        and hashes. If a match is found it refers to local instance
        instead. If it is not, the image is fetched, created and
        referenced.
        '''
        image_detail_url = "{}{}/".format(self.image_url, image_id)

        try:
            img_response = requests.get(image_detail_url)
            img_info = json.loads(img_response.content)
        except Exception as e:
            error_context = {
                "image detail url": image_detail_url,
                "exception": e,
            }
            raise ImageInfoFetchFailed(error_context)

        if img_info["image_hash"] is None:
            raise ValueError('image hash should not be none')

        # check if a replica exists
        local_image = self.get_replica_image(
            img_info["image_hash"])

        file_url = img_info['image_url']

        # handle when image_url is relative
        # assumes that image import means local storage
        if img_info['image_url'][0] == '/':
            file_url = "{}{}".format(
                self.base_url, img_info['image_url'])

        if local_image:
            context = {
                "local_version_existed": True,
                "file_url": file_url,
                "image_detail_url": image_detail_url,
                "foreign_title": img_info["title"].encode('utf-8'),
            }
            # update record keeper
            if self.record_keeper:
                self.record_keeper.record_image_relation(
                    image_id,
                    local_image.id)
            return (local_image, context)
        else:
            new_image, context = self.fetch_and_create_image(
                file_url,
                img_info["title"].encode('utf-8'))
            # update record keeper
            if self.record_keeper:
                self.record_keeper.record_image_relation(
                    image_id,
                    new_image.id)
            context.update({
                "local_version_existed": False,
            })
            return (new_image, context)

    def import_images(self):
        '''
        Fetches all images from site

        Handles Errors in creation process
        Updates record_keeper
        Logs the result of each attempt to create an image
        '''
        self.log(ACTION, "Importing Images")
        try:
            images = list_of_objects_from_api(self.image_url)
        except Exception as e:
            raise ImageInfoFetchFailed(
                "Something went wrong fetching list of images")

        if not images:
            return None

        # iterate through foreign images
        for image_summary in images:
            self.log(ACTION, "Importing Image", depth=1)
            try:
                (image, context) = self.import_image(image_summary["id"])
                # log success
                self.log(SUCCESS, "Importing Image",
                         context=context,
                         depth=1)
            except ImageInfoFetchFailed as e:
                self.log(ERROR, "Importing Images", e, depth=1)
            except ImageCreationFailed as e:
                self.log(ERROR, "Importing Images", e.message, depth=1)
            except Exception as e:
                context = {
                    "exception": e,
                    "foreign_image_id": image_summary["id"],
                }
                self.log(ERROR, "Importing Images", context, depth=1)


class LanguageImporter(BaseImporter):
    def __init__(self, site_pk, base_url, record_keeper=None, logger=None):
        super(LanguageImporter, self).__init__(site_pk, base_url,
                                               record_keeper=record_keeper,
                                               logger=logger)
        self.language_url = "{}languages/".format(self.api_url)

    def get_language_ids(self):
        '''
        Return list of foreign language IDs from API language endpoint

        TODO: add in validation before creating languages
        '''
        languages = list_of_objects_from_api(self.language_url)

        language_ids = []
        for language in languages:
            language_ids.append(language["id"])
        return language_ids

    def copy_site_languages(self):
        language_foreign_ids = self.get_language_ids()
        language_setting, created = Languages.objects.get_or_create(
            site_id=self.site_pk)
        for foreign_id in language_foreign_ids:
            language_url = "{}{}/".format(self.language_url,
                                          foreign_id)
            response = requests.get(language_url)
            content = json.loads(response.content)

            sle, created = SiteLanguageRelation.objects.get_or_create(
                locale=content['locale'],
                is_active=content['is_active'],
                language_setting=language_setting)


class ContentImporter(BaseImporter):
    def __init__(self, site_pk, base_url, record_keeper=None, logger=None):
        super(ContentImporter, self).__init__(site_pk, base_url,
                                              record_keeper=record_keeper,
                                              logger=logger)

    def recreate_relationships(self, class_, attribute_name, key):
        '''
        Recreates one-to-many relationship
        '''
        iterable = self.record_keeper.foreign_to_many_foreign_map[key]
        for foreign_page_id, foreign_page_id_list in iterable.iteritems():

            # Assumption: local page has been indexed and exists
            # TODO: handle case where it doesn't exist
            local_page_id = self.record_keeper.get_local_page(foreign_page_id)
            local_page = Page.objects.get(id=local_page_id).specific

            for _foreign_page_id in foreign_page_id_list:
                try:
                    local_version_page_id = (self.record_keeper
                                             .get_local_page(_foreign_page_id))
                    foreign_page = Page.objects.get(
                        id=local_version_page_id).specific
                    realtionship_object = class_(page=local_page)
                    setattr(realtionship_object, attribute_name, foreign_page)
                    realtionship_object.save()
                except Exception as e:
                    context = {
                        "exception": e,
                        "function_schema": ("recreate_relationships"
                                            "(class, attribute_name, key)"),
                        "attribute_name": str(attribute_name),
                        "key": str(key),
                        "class": str(class_),
                        "foreign_page_id": str(foreign_page_id),
                    }
                    self.log(ERROR, "recreating relationships",
                             context=context)

    def recreate_relationship(self, attribute_name, key):
        '''
        Recreates one-to-one relationship
        '''
        iterator = self.record_keeper.foreign_to_foreign_map["banner_link_page"].iteritems()  # noqa
        for foreign_page_id, linked_page_foreign_id in iterator:
            # get local banner page
            local_page_id = self.record_keeper.get_local_page(foreign_page_id)
            local_page = Page.objects.get(id=local_page_id).specific

            # get local linked page
            local_id = self.record_keeper.get_local_page(
                linked_page_foreign_id)
            linked_page = Page.objects.get(id=local_id).specific

            # link the two together
            setattr(local_page, attribute_name, linked_page)
            # TODO: review publishing and saving revisions
            local_page.save_revision().publish()

    def create_banner_page_links(self):
        self.recreate_relationship("banner_link_page", "banner_link_page")

    def create_recommended_articles(self):
        self.recreate_relationships(
            ArticlePageRecommendedSections,
            'recommended_article',
            'recommended_articles',
        )

    def create_related_sections(self):
        self.recreate_relationships(
            ArticlePageRelatedSections,
            'section',
            'related_sections',
        )

    def create_nav_tag_relationships(self):
        self.recreate_relationships(
            ArticlePageTags,
            'tag',
            'nav_tags'
        )

    def create_section_tag_relationship(self):
        self.recreate_relationships(
            SectionPageTags,
            'tag',
            'section_tags'
        )

    def recreate_article_body(self):
        '''
        Handles case where article body contained page or image.

        Assumes all articles and images have been created.
        '''
        for foreign_id, body in self.record_keeper.article_bodies.iteritems():
            try:
                local_page_id = self.record_keeper.get_local_page(foreign_id)
                page = Page.objects.get(id=local_page_id).specific

                # iterate through the body
                new_body = []
                for item in body:
                    if not item['value']:
                        continue
                    if item['type'] == 'page':
                        new_page_id = self.record_keeper.get_local_page(
                            item['value'])
                        item['value'] = new_page_id
                    elif item['type'] == 'image':
                        new_image_id = self.record_keeper.get_local_image(
                            item['value'])
                        item['value'] = new_image_id

                    new_body.append(item)

                setattr(page, 'body', json.dumps(new_body))
                page.save_revision().publish()

            except Exception as e:
                self.log(ERROR, "recreating article body",
                         {
                             "exception": e,
                             "foreign_id": foreign_id,
                             "body": body,
                         },
                         depth=1)

    def get_foreign_page_id_from_type(self, page_type):
        '''
        Get the foreign page id based on type

        Only works for index pages
        '''
        # TODO: log this
        response = requests.get("{}pages/?type={}".format(
            self.api_url, page_type))
        content = json.loads(response.content)
        return content["items"][0]["id"]

    def attach_page(self, parent, content):
        page_type = content["meta"]["type"].split(".")[-1]
        class_ = eval(page_type)
        content_copy = dict(content)

        if not issubclass(class_, ImportableMixin):
            error_context = {
                "message": "Page does not inherit ImportableMixin",
                "page type": page_type,
                "class_": str(class_),
                "outcome": "cannot import page or its descendants"}
            raise PageNotImportable(error_context)

        # TODO: make a copy of the record_keeper class

        try:
            page = class_.create_page(
                content_copy, class_, record_keeper=self.record_keeper,
                logger=self.logger)
        except Exception as e:
            # avoid side-effects of adding the page
            # TODO: restore copy of record Keeper class
            error_context = {
                "message": "ERROR: Creating Page",
                "exception": e,
                "content": content,
                "outcome": "cannot import page or its descendants",
                "foreign_page_id": content["id"]}
            raise ImportedContentInvalid(error_context, None)

        try:
            parent.add_child(instance=page)
            parent.save_revision().publish()
            page.save_revision().publish()
        except Exception as e:
            error_context = {
                "message": "ERROR: Saving Page",
                "exception": e,
                "outcome": "page and its descendants will not be imported",
                "foreign_page_id": content["id"]}
            raise ImportedPageNotSavable(error_context)

        self.record_keeper.record_page_relation(
            content["id"],
            page.id
        )
        # TODO: raise to calling function
        # self.log(SUCCESS, "Page imported", {
        #         "parent title": parent.title, "page title": page.title})
        return page

    def attach_translated_content(self, local_main_lang_page,
                                  content, locale):
        '''
        Wrapper for  attach_page

        Creates the content
        Then attaches a language relation from the main language page to the
        newly created Page
        Note: we get the parent from the main language page
        '''
        try:
            page = self.attach_page(
                local_main_lang_page.get_parent(),
                content)
        except:
            # TODO: log this
            return None

        try:
            # create the translation object for page
            language = SiteLanguageRelation.objects.get(
                language_setting=self.language_setting,
                locale=locale)
            language_relation = page.languages.first()
            language_relation.language = language
            language_relation.save()
            PageTranslation.objects.get_or_create(
                page=local_main_lang_page,
                translated_page=page)
        except:
            # TODO: log that creating translation failed
            # TODO: log that page is now being deleted
            page.delete()
        return page

    def copy_page_and_children(self, foreign_id, parent_id, depth=0):
        '''
        Recusively copies over pages, their translations, and child pages
        '''
        url = "{}/api/v2/pages/{}/".format(self.base_url, foreign_id)

        self.log(ACTION, "Requesting Data", {"url": url}, depth)
        try:
            # TODO: create a robust wrapper around this functionality
            response = requests.get(url)
            content = json.loads(response.content)
        except Exception as e:
            self.log(ERROR, "Requesting Data - abandoning copy",
                     {"url": url, "exception": e}, depth)
            return None

        parent = Page.objects.get(id=parent_id).specific
        page = None
        try:
            self.log(ACTION, "Create Page", {"url": url}, depth)
            page = self.attach_page(parent, content)
            if page:
                self.log(SUCCESS, "Create Page",
                         {"url": url,
                          "page title": page.title.encode('utf-8')},
                         depth)
        except PageNotImportable as e:
            message = e.message.pop("message")
            self.log(WARNING, message, e.message.pop("message"), depth)
            return None

        '''
        except Exception as e:
            context = {
                "exception": e,
                "url": url,
                "parent title": parent.title,
                "foreign_page": content["title"].encode('utf-8'),
            }
            self.log(
                ERROR, "Create Page - abandon page and children creation",
                context, depth)
        '''

        if page:
            # create translations
            if content["meta"]["translations"]:
                for translation_obj in content["meta"]["translations"]:
                    _url = "{}/api/v2/pages/{}/".format(self.base_url,
                                                        translation_obj["id"])
                    # TODO: create a robust wrapper around this functionality
                    _response = requests.get(_url)
                    self.log(
                        ACTION,
                        "Getting translated content",
                        {"url": _url}, depth)
                    if _response.content:
                        _content = json.loads(_response.content)

                        if ("locale" in translation_obj and
                                translation_obj["locale"]):
                            self.attach_translated_content(
                                page, _content, translation_obj["locale"])
                        else:
                            self.log(
                                ERROR,
                                "locale is null",
                                {"url": _url, }, depth)
                    else:
                        self.log(
                            ERROR,
                            "Getting translated content",
                            {"url": _url}, depth)

            main_language_child_ids = content["meta"]["main_language_children"]

            # recursively iterate through child nodes
            if main_language_child_ids:
                for main_language_child_id in main_language_child_ids:
                    try:
                        self.copy_page_and_children(
                            foreign_id=main_language_child_id,
                            parent_id=page.id, depth=depth + 1)
                    except Exception as e:
                        self.log(ERROR, "Copying Children",
                                 {"url": url, "exception": e})

    def copy_children(self, foreign_id, existing_node):
        '''
        Initiates copying of tree, with existing_node acting as root
        '''
        url = "{}/api/v2/pages/{}/".format(self.base_url, foreign_id)
        self.log(
            ACTION,
            "Copying Children",
            {"existing node type": str(type(existing_node))})

        # TODO: create a robust wrapper around this functionality
        try:
            self.log(ACTION, "Requesting Data", {"url": url})
            response = requests.get(url)
            content = json.loads(response.content)
            self.log(SUCCESS, "Data Fetched Successfully", {"url": url})

            main_language_child_ids = content["meta"]["main_language_children"]
            if main_language_child_ids:
                for main_language_child_id in main_language_child_ids:
                    self.copy_page_and_children(
                        foreign_id=main_language_child_id,
                        parent_id=existing_node.id, depth=1)
            else:
                self.log(SUCCESS, "No children to copy")
        except Exception as e:
            self.log(ERROR, "Copying Children", {"url": url, "exception": e})

    def restore_relationships(self):
        pass


class Logger(object):
    def __init__(self):
        self.record = []

    def format_message(self, log_type, message, context, depth=0):
        log_item = "{}>> {}: {}".format(depth * "\t", log_type, message)
        if context:
            for key, item in context.iteritems():
                if key == "exception":
                    log_item += "\n{}| {}: {}".format(depth * "\t", "ex_type",
                                                      type(item))
                    log_item += "\n{}| {}: {}".format(depth * "\t", key, item)
                else:
                    log_item += (
                        "\n{}| {}: {}".format(depth * "\t", key, item))
            log_item += "\n{}----\n".format(depth * "\t")
        return log_item

    def get_email_logs(self):
        '''
        Returns a string representation of logs.

        Only displays errors and warnings in the email logs
        to avoid being verbose
        '''
        message = ""
        for log in self.record:
            if log["log_type"] in [ERROR, WARNING]:
                message += self.format_message(**log)

        return message

    def log(self, log_type, message, context=None, depth=0):
        log_message = self.format_message(
            log_type, message, context, depth=depth)

        logger = logging.getLogger('import_logger')
        logger.debug(log_message)

        self.record.append({
            "log_type": log_type,
            "message": message,
            "context": context,
        })



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0031_sitesettings'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='ga_tag_manager',
            field=models.CharField(choices=[(b'af', b'Afrikaans'), (b'ar', b'Arabic'), (b'ast', b'Asturian'), (b'az', b'Azerbaijani'), (b'bg', b'Bulgarian'), (b'be', b'Belarusian'), (b'bn', b'Bengali'), (b'br', b'Breton'), (b'bs', b'Bosnian'), (b'ca', b'Catalan'), (b'cs', b'Czech'), (b'cy', b'Welsh'), (b'da', b'Danish'), (b'de', b'German'), (b'el', b'Greek'), (b'en', b'English'), (b'en-au', b'Australian English'), (b'en-gb', b'British English'), (b'eo', b'Esperanto'), (b'es', b'Spanish'), (b'es-ar', b'Argentinian Spanish'), (b'es-mx', b'Mexican Spanish'), (b'es-ni', b'Nicaraguan Spanish'), (b'es-ve', b'Venezuelan Spanish'), (b'et', b'Estonian'), (b'eu', b'Basque'), (b'fa', b'Persian'), (b'fi', b'Finnish'), (b'fr', b'French'), (b'fy', b'Frisian'), (b'ga', b'Irish'), (b'gl', b'Galician'), (b'he', b'Hebrew'), (b'hi', b'Hindi'), (b'hr', b'Croatian'), (b'hu', b'Hungarian'), (b'ia', b'Interlingua'), (b'id', b'Indonesian'), (b'io', b'Ido'), (b'is', b'Icelandic'), (b'it', b'Italian'), (b'ja', b'Japanese'), (b'ka', b'Georgian'), (b'kk', b'Kazakh'), (b'km', b'Khmer'), (b'kn', b'Kannada'), (b'ko', b'Korean'), (b'lb', b'Luxembourgish'), (b'lt', b'Lithuanian'), (b'lv', b'Latvian'), (b'mk', b'Macedonian'), (b'ml', b'Malayalam'), (b'mn', b'Mongolian'), (b'mr', b'Marathi'), (b'my', b'Burmese'), (b'nb', b'Norwegian Bokmal'), (b'ne', b'Nepali'), (b'nl', b'Dutch'), (b'nn', b'Norwegian Nynorsk'), (b'os', b'Ossetic'), (b'pa', b'Punjabi'), (b'pl', b'Polish'), (b'pt', b'Portuguese'), (b'pt-br', b'Brazilian Portuguese'), (b'ro', b'Romanian'), (b'ru', b'Russian'), (b'sk', b'Slovak'), (b'sl', b'Slovenian'), (b'sq', b'Albanian'), (b'sr', b'Serbian'), (b'sr-latn', b'Serbian Latin'), (b'sv', b'Swedish'), (b'sw', b'Swahili'), (b'ta', b'Tamil'), (b'te', b'Telugu'), (b'th', b'Thai'), (b'tr', b'Turkish'), (b'tt', b'Tatar'), (b'udm', b'Udmurt'), (b'uk', b'Ukrainian'), (b'ur', b'Urdu'), (b'vi', b'Vietnamese'), (b'zh-cn', b'Simplified Chinese'), (b'zh-hans', b'Simplified Chinese'), (b'zh-hant', b'Traditional Chinese'), (b'zh-tw', b'Traditional Chinese'), (b'zu', 'Zulu'), (b'xh', 'Xhosa'), (b'st', 'Sotho'), (b've', 'Venda'), (b'tn', 'Tswana'), (b'ts', 'Tsonga'), (b'ss', 'Swati'), (b'nr', 'Ndebele')], max_length=255, blank=True, help_text='GA Tag Manager tracking code (e.g GTM-XXX)', null=True, verbose_name='GA Tag Manager'),
        ),
    ]



from rest_framework.filters import BaseFilterBackend


class MainLanguageFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        """
        Returns only pages in the main language for a site
        """
        if 'is_main_language' in request.GET:
            # TODO investigate possible error cases where page
            # does not have language
            return queryset.filter(languages__language__is_main_language=True)
        else:
            return queryset
